@@ -196,6 +196,7 @@ except TypeError:
 
 noitem = object()
 
+
 def item(iterator, index, default=noitem):
     """
     Return the ``index``th item from the iterator ``iterator``.
@@ -670,6 +671,12 @@ def xrepr_none(self, mode="default"):
 xrepr.when_object(None)(xrepr_none)
 
 
+def xrepr_noitem(self, mode="default"):
+    yield (2, True)
+    yield (astyle.style_nodata, "<?>")
+xrepr.when_object(noitem)(xrepr_noitem)
+
+
 def xrepr_bool(self, mode="default"):
     yield (astyle.style_type_bool, repr(self))
 xrepr.when_type(bool)(xrepr_bool)
@@ -943,7 +950,13 @@ def xiter(item):
         elif _isstr(item):
             if not item:
                 raise ValueError("can't enter empty string")
-            return iter(item.splitlines())
+            lines = item.splitlines()
+            if len(lines) == 1:
+               def iterone(item):
+                   yield item
+               return iterone(item)
+            else:
+                return iter(lines)
         return iter(item)
     else:
         return iter(func()) # iter() just to be safe
@@ -2089,8 +2102,15 @@ class AttributeDetail(Table):
 try:
     from ibrowse import ibrowse
 except ImportError:
-    # No curses (probably Windows) => use ``idump`` as the default display.
-    defaultdisplay = idump
+    # No curses (probably Windows)
+    try:
+        from igrid import igrid
+    except ImportError:
+        # no wx eithevn do => use ``idump`` as the default display.
+        defaultdisplay = idump
+    else:
+        defaultdisplay = igrid
+        __all__.append("igrid")
 else:
     defaultdisplay = ibrowse
     __all__.append("ibrowse")
