@@ -5,7 +5,7 @@ General purpose utilities.
 This is a grab-bag of stuff I find useful in most programs I write. Some of
 these things are also convenient when working at the command line.
 
-$Id: genutils.py 1349 2006-06-04 00:57:43Z fperez $"""
+$Id: genutils.py 2190 2007-03-30 18:35:46Z fperez $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -24,11 +24,13 @@ import __main__
 import commands
 import os
 import re
+import shlex
 import shutil
 import sys
 import tempfile
 import time
 import types
+import warnings
 
 # Other IPython utilities
 from IPython.Itpl import Itpl,itpl,printpl
@@ -65,6 +67,10 @@ class IOStream:
                 # if we get here, something is seriously broken.
                 print >> sys.stderr, \
                       'ERROR - failed to write data to stream:', self.stream
+
+    def close(self):
+        pass
+                
         
 class IOTerm:
     """ Term holds the file or file-like objects for handling I/O operations.
@@ -161,15 +167,33 @@ StringTypes = types.StringTypes
 # If possible (Unix), use the resource module instead of time.clock()
 try:
     import resource
+    def clocku():
+        """clocku() -> floating point number
+
+        Return the *USER* CPU time in seconds since the start of the process.
+        This is done via a call to resource.getrusage, so it avoids the
+        wraparound problems in time.clock()."""
+        
+        return resource.getrusage(resource.RUSAGE_SELF)[0]
+    
+    def clocks():
+        """clocks() -> floating point number
+
+        Return the *SYSTEM* CPU time in seconds since the start of the process.
+        This is done via a call to resource.getrusage, so it avoids the
+        wraparound problems in time.clock()."""
+        
+        return resource.getrusage(resource.RUSAGE_SELF)[1]
+    
     def clock():
         """clock() -> floating point number
 
-        Return the CPU time in seconds (user time only, system time is
-        ignored) since the start of the process.  This is done via a call to
-        resource.getrusage, so it avoids the wraparound problems in
-        time.clock()."""
-        
-        return resource.getrusage(resource.RUSAGE_SELF)[0]
+        Return the *TOTAL USER+SYSTEM* CPU time in seconds since the start of
+        the process.  This is done via a call to resource.getrusage, so it
+        avoids the wraparound problems in time.clock()."""
+
+        u,s = resource.getrusage(resource.RUSAGE_SELF)[:2] 
+        return u+s
     
     def clock2():
         """clock2() -> (t_user,t_system)
@@ -178,7 +202,9 @@ try:
         return resource.getrusage(resource.RUSAGE_SELF)[:2]
 
 except ImportError:
-    clock = time.clock
+    # There is no distinction of user/system time under windows, so we just use
+    # time.clock() for everything...
+    clocku = clocks = clock = time.clock
     def clock2():
         """Under windows, system CPU time can't be measured.
 
@@ -233,6 +259,24 @@ def timing(func,*args,**kw):
 #****************************************************************************
 # file and system
 
+def arg_split(s,posix=False):
+    """Split a command line's arguments in a shell-like manner.
+
+    This is a modified version of the standard library's shlex.split()
+    function, but with a default of posix=False for splitting, so that quotes
+    in inputs are respected."""
+
+    # XXX - there may be unicode-related problems here!!!  I'm not sure that
+    # shlex is truly unicode-safe, so it might be necessary to do
+    #
+    # s = s.encode(sys.stdin.encoding)
+    #
+    # first, to ensure that shlex gets a normal string.  Input from anyone who
+    # knows more about unicode and shlex than I would be good to have here...
+    lex = shlex.shlex(s, posix=posix)
+    lex.whitespace_split = True
+    return list(lex)
+
 def system(cmd,verbose=0,debug=0,header=''):
     """Execute a system command, return its exit status.
 
@@ -1214,7 +1258,8 @@ def get_pager_start(pager,start):
     return start_string
 
 #----------------------------------------------------------------------------
-if os.name == "nt":
+# (X)emacs on W32 doesn't like to be bypassed with msvcrt.getch()
+if os.name == 'nt' and os.environ.get('TERM','dumb') != 'emacs':
     import msvcrt
     def page_more():
         """ Smart pausing between pages
@@ -1685,6 +1730,7 @@ def import_fail_info(mod_name,fns=None):
 #----------------------------------------------------------------------------
 # Proposed popitem() extension, written as a method
 
+
 class NotGiven: pass
 
 def popkey(dct,key,default=NotGiven):
@@ -1703,5 +1749,15 @@ def popkey(dct,key,default=NotGiven):
     else:
         del dct[key]
         return val
+
+def wrap_deprecated(func, suggest = '<nothing>'):
+    def newFunc(*args, **kwargs):
+        warnings.warn("Call to deprecated function %s, use %s instead" % 
+                      ( func.__name__, suggest),
+                      category=DeprecationWarning,
+                      stacklevel = 2)
+        return func(*args, **kwargs)
+    return newFunc
+    
 #*************************** end of file <genutils.py> **********************
 
