@@ -91,6 +91,12 @@ class IPythonNotRunning:
     without ipython.  Obviously code which uses the ipython object for
     computations will not work, but this allows a wider range of code to
     transparently work whether ipython is being used or not."""
+
+    def __init__(self,warn=True):
+        if warn:
+            self.dummy = self._dummy_warn
+        else:
+            self.dummy = self._dummy_silent
     
     def __str__(self):
         return "<IPythonNotRunning>"
@@ -100,26 +106,33 @@ class IPythonNotRunning:
     def __getattr__(self,name):
         return self.dummy
 
-    def dummy(self,*args,**kw):
+    def _dummy_warn(self,*args,**kw):
         """Dummy function, which doesn't do anything but warn."""
+
         warn("IPython is not running, this is a dummy no-op function")
 
+    def _dummy_silent(self,*args,**kw):
+        """Dummy function, which doesn't do anything and emits no warnings."""
+        pass
+
 _recent = None
 
 
-def get(allow_dummy=False):
+def get(allow_dummy=False,dummy_warn=True):
     """Get an IPApi object.
 
     If allow_dummy is true, returns an instance of IPythonNotRunning 
     instead of None if not running under IPython.
 
+    If dummy_warn is false, the dummy instance will be completely silent.
+
     Running this should be the first thing you do when writing extensions that
     can be imported as normal modules. You can then direct all the
     configuration operations against the returned object.
     """
     global _recent
     if allow_dummy and not _recent:
-        _recent = IPythonNotRunning()
+        _recent = IPythonNotRunning(dummy_warn)
     return _recent
 
 class IPApi:
@@ -275,6 +288,21 @@ class IPApi:
                 error('could not get var. %s from %s' %
                 (name,cf.f_code.co_name))
 
+    def expand_alias(self,line):
+        """ Expand an alias in the command line 
+        
+        Returns the provided command line, possibly with the first word 
+        (command) translated according to alias expansion rules.
+        
+        [ipython]|16> _ip.expand_aliases("np myfile.txt")
+                 <16> 'q:/opt/np/notepad++.exe myfile.txt'
+        """
+        
+        pre,fn,rest = self.IP.split_user_input(line)
+        res = pre + self.IP.expand_aliases(fn,rest)
+        
+        
+
 def launch_new_instance(user_ns = None):
     """ Make and start a new ipython instance.
     
