@@ -6,7 +6,7 @@ Requires Python 2.1 or better.
 
 This file contains the main make_IPython() starter function.
 
-$Id: ipmaker.py 1979 2006-12-12 18:50:20Z vivainio $"""
+$Id: ipmaker.py 2156 2007-03-19 02:32:19Z fperez $"""
 
 #*****************************************************************************
 #       Copyright (C) 2001-2006 Fernando Perez. <fperez@colorado.edu>
@@ -20,17 +20,22 @@ __author__  = '%s <%s>' % Release.authors['Fernando']
 __license__ = Release.license
 __version__ = Release.version
 
-credits._Printer__data = """
-Python: %s
+try:
+    credits._Printer__data = """
+    Python: %s
 
-IPython: Fernando Perez, Janko Hauser, Nathan Gray, and many users.
-See http://ipython.scipy.org for more information.""" \
-% credits._Printer__data
+    IPython: Fernando Perez, Janko Hauser, Nathan Gray, and many users.
+    See http://ipython.scipy.org for more information.""" \
+    % credits._Printer__data
 
-copyright._Printer__data += """
+    copyright._Printer__data += """
 
-Copyright (c) 2001-2004 Fernando Perez, Janko Hauser, Nathan Gray.
-All Rights Reserved."""
+    Copyright (c) 2001-2004 Fernando Perez, Janko Hauser, Nathan Gray.
+    All Rights Reserved."""
+except NameError:
+    # Can happen if ipython was started with 'python -S', so that site.py is
+    # not loaded
+    pass
 
 #****************************************************************************
 # Required modules
@@ -154,6 +159,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                     'debug! deep_reload! editor=s log|l messages! nosep '
                     'object_info_string_level=i pdb! '
                     'pprint! prompt_in1|pi1=s prompt_in2|pi2=s prompt_out|po=s '
+                    'pylab_import_all! '
                     'quick screen_length|sl=i prompts_pad_left=i '
                     'logfile|lf=s logplay|lp=s profile|p=s '
                     'readline! readline_merge_completions! '
@@ -162,7 +168,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                     'separate_out2|so2=s xmode=s wildcards_case_sensitive! '
                     'magic_docstrings system_verbose! '
                     'multi_line_specials! '
-                    'wxversion=s '
+                    'term_title! wxversion=s '
                     'autoedit_syntax!')
 
     # Options that can *only* appear at the cmd line (not in rcfiles).
@@ -208,6 +214,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       prompt_in2 = '   .\\D.: ',
                       prompt_out = 'Out[\\#]: ',
                       prompts_pad_left = 1,
+                      pylab_import_all = 1,
                       quiet = 0,
                       quick = 0,
                       readline = 1,
@@ -225,6 +232,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                       q4thread = 0,
                       wthread = 0,
                       pylab = 0,
+                      term_title = 1,
                       tk = 0,
                       upgrade = 0,
                       Version = 0,
@@ -431,8 +439,6 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
     rcfiledata = None
     if opts_all.quick:
         print 'Launching IPython in quick mode. No config file read.'
-    elif opts_all.classic:
-        print 'Launching IPython in classic mode. No config file read.'
     elif opts_all.rcfile:
         try:
             cfg_loader = ConfigLoader(conflict)
@@ -627,8 +633,8 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
                 warn("Could not start with profile '%s'!\n"
                      "('%s/%s.py' does not exist? run '%%upgrade')" %
                      (opts_all.profile, opts_all.ipythondir, profmodname) )
-        except:
-            print "Error importing",profmodname
+        except:            
+            print "Error importing",profmodname,"- perhaps you should run %upgrade?"
             IP.InteractiveTB()
             import_fail_info(profmodname)
                       
@@ -640,7 +646,7 @@ object? -> Details about 'object'. ?object also works, ?? prints more.
              "('%s/ipy_user_conf.py' does not exist? Please run '%%upgrade')\n"
              % opts_all.ipythondir)
     except:
-        print "Error importing ipy_user_conf"
+        print "Error importing ipy_user_conf - perhaps you should run %upgrade?"
         IP.InteractiveTB()
         import_fail_info("ipy_user_conf")
     
